pro biggest_box, bmask, x0, y0, nx, ny , $
     pixpad=pixpad, tolerence=tolerence, missing=missing, display=display
;+
;   Name: biggest_box
;
;   Purpose: find the largest 'valid data' rectangular area 
;
;   Input Parameters:
;      bmask - assumed irregular shaped ROI (data or boolean mask)
;
;   Keyword Parameters:
;      pixpad - number of pixels to pad (reduced by this; "more conservative")
;      missing - value of "missing data" (default = 0)
;      display - if set, show some stuff
;      tolerence - if set, number of pixels slop allowed (default=2)
;
;   Calling Sequence:
;      IDL> biggest_box, data, x,y,nx,ny [,pixpad=npix, tolerence=npix ,/display]
;
;   Method: find largest contiguous, non-missing area (irregular)
;           then apply brute force - I'll vectorize when I have some time...
;
;   16-Oct-2000 - S.L.Freeland  - a potential pre-processor for automating
;                 and applying existing auto-correlation software
;                 
;   Motivation: lots of existing auto-corrlation software accepts
;      rectangular subregion for the auto correlation - this routine applied
;      to the best guess overlapping pixel arrays (boolean mask for example)
;      can help auto-select the "best" subregion to apply the subsequent
;      correlation (korrel.pro, correl_optimize,...)
;
;  Restrictions:
;     Current rev finds largest square (nx=ny) 
;     TODO - find largest rectangle (allow independent nx/ny)
;
;  Side Effects:
;    if /DISPLAY set, make a window and loads a color table (linecolors.pro)
;-
;
if data_chk(bmask,/nimage) ne 1 then begin 
   box_message, 'Need a 2D image...
   return
endif

display=keyword_set(display)
if n_elements(missing) eq 0 then missing=0

regions=label_region(bmask)
rhist=histogram(regions)
rhist=rem_elem(rhist,missing,hcnt)            ; ignore missing region 

if hcnt eq 0 then begin
   box_message,'All data is MISSING'
   return
endif

rss=where(regions eq max(rhist),sscnt)

template=make_array(size=size(bmask))
template(rss)=1
centroidw, template, x, y
xy=coord_l2v(rss,size(template))

if not keyword_set(tolerence) then tolerence=2   ; pixels per step
npix=tolerence>2
nmiss=0

if display then begin               ; optionally init display 
   linecolors
   wdef,im=template
   tvscl,template
endif

plim=min([data_chk(template,/x),data_chk(template,/y)])

while nmiss eq 0 and npix le plim do begin 
   arr=extract_arr(template, x, npix, y, npix)
   ssmiss=where(arr eq 0,nmiss)
   if display then draw_boxcensiz, x, y, npix, npix,/dev,color=2
   npix=npix+tolerence
endwhile
npix=npix-tolerence

if n_elements(pixpad) eq 0 then pixpad=0
npix=npix-pixpad

x0=fix(x-npix/2)>0
y0=fix(y-npix/2)>0
nx=npix<data_chk(template,/nx)
ny=npix<data_chk(template,/ny)

if display then begin 
   draw_boxcensiz,x0+nx/2,y0+ny/2,nx,ny,color=7,/dev    ; show final box
endif

return
end




